'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var snakeCase = _interopDefault(require('snake-case'));
var qs = _interopDefault(require('qs'));
var hash = _interopDefault(require('object-hash'));
var fetch = _interopDefault(require('isomorphic-unfetch'));

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
function __awaiter(thisArg, _arguments, P, generator) {
  return new (P || (P = Promise))(function (resolve, reject) {
    function fulfilled(value) {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    }

    function rejected(value) {
      try {
        step(generator["throw"](value));
      } catch (e) {
        reject(e);
      }
    }

    function step(result) {
      result.done ? resolve(result.value) : new P(function (resolve) {
        resolve(result.value);
      }).then(fulfilled, rejected);
    }

    step((generator = generator.apply(thisArg, _arguments || [])).next());
  });
}
function __generator(thisArg, body) {
  var _ = {
    label: 0,
    sent: function () {
      if (t[0] & 1) throw t[1];
      return t[1];
    },
    trys: [],
    ops: []
  },
      f,
      y,
      t,
      g;
  return g = {
    next: verb(0),
    "throw": verb(1),
    "return": verb(2)
  }, typeof Symbol === "function" && (g[Symbol.iterator] = function () {
    return this;
  }), g;

  function verb(n) {
    return function (v) {
      return step([n, v]);
    };
  }

  function step(op) {
    if (f) throw new TypeError("Generator is already executing.");

    while (_) try {
      if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
      if (y = 0, t) op = [op[0] & 2, t.value];

      switch (op[0]) {
        case 0:
        case 1:
          t = op;
          break;

        case 4:
          _.label++;
          return {
            value: op[1],
            done: false
          };

        case 5:
          _.label++;
          y = op[1];
          op = [0];
          continue;

        case 7:
          op = _.ops.pop();

          _.trys.pop();

          continue;

        default:
          if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
            _ = 0;
            continue;
          }

          if (op[0] === 3 && (!t || op[1] > t[0] && op[1] < t[3])) {
            _.label = op[1];
            break;
          }

          if (op[0] === 6 && _.label < t[1]) {
            _.label = t[1];
            t = op;
            break;
          }

          if (t && _.label < t[2]) {
            _.label = t[2];

            _.ops.push(op);

            break;
          }

          if (t[2]) _.ops.pop();

          _.trys.pop();

          continue;
      }

      op = body.call(thisArg, _);
    } catch (e) {
      op = [6, e];
      y = 0;
    } finally {
      f = t = 0;
    }

    if (op[0] & 5) throw op[1];
    return {
      value: op[0] ? op[1] : void 0,
      done: true
    };
  }
}

var normalize = function (_uri, params) {
    var uri = _uri;
    Object.keys(params).forEach(function (key) {
        if (uri.match("\\[" + key + "\\]")) {
            uri = uri.replace(new RegExp("\\[" + key + "\\]", 'g'), encodeURIComponent(params[key]));
            delete params[key];
        }
    });
    return uri;
};
var queryHeader = function (headers, defaultHeaders, mode, credentials) { return ({
    method: 'GET',
    headers: Object.assign({}, defaultHeaders, headers),
    mode: mode,
    credentials: credentials
}); };
var commandHeader = function (method, values, headers, defaultHeaders, mode, credentials, useQuery) { return ({
    method: method,
    headers: Object.assign({
        'Content-Type': 'application/json'
    }, defaultHeaders, headers),
    body: !useQuery ? JSON.stringify(values) : '',
    mode: mode,
    credentials: credentials
}); };
var toObject = function (headers) {
    var obj = {};
    headers.forEach(function (value, key) {
        obj[key] = value;
    });
    return obj;
};
var ApiClient = /** @class */ (function () {
    function ApiClient(_a) {
        var _b = _a.defaultHeaders, defaultHeaders = _b === void 0 ? {} : _b, _c = _a.logger, logger = _c === void 0 ? function (args) { return console.log(args); } : _c;
        var cached = {};
        this.fetch = function (_a) {
            var _b = _a.url, url = _b === void 0 ? '' : _b, _c = _a.method, method = _c === void 0 ? 'GET' : _c, _d = _a.values, values = _d === void 0 ? {} : _d, _e = _a.headers, headers = _e === void 0 ? {} : _e, mode = _a.mode, credentials = _a.credentials, cache = _a.cache, useQuery = _a.useQuery;
            if (!url) {
                throw new Error('URL does not specified');
            }
            var _values = Object.assign({}, values);
            var _url = normalize(url, _values) +
                (useQuery || method === 'GET' ? "?" + qs.stringify(_values) : '');
            // return from cache
            var hashed = hash(_url);
            if (cache && method === 'GET' && cached[hashed]) {
                logger("## return from cache " + _url + " " + method);
                return Promise.resolve(cached[hashed]);
            }
            return new Promise(function (resolve, reject) {
                logger("## fetch " + _url + " " + method);
                var config = method === 'GET'
                    ? queryHeader(headers, defaultHeaders, mode, credentials)
                    : commandHeader(method, _values, headers, defaultHeaders, mode, credentials, useQuery);
                fetch(_url, config).then(function (res) {
                    if (!res.ok) {
                        res.json().then(function (json) {
                            reject({
                                body: json,
                                headers: toObject(res.headers),
                                status: res.status,
                                ok: res.ok
                            });
                        }, function () {
                            reject({
                                body: {},
                                headers: toObject(res.headers),
                                status: res.status,
                                ok: res.ok
                            });
                        });
                    }
                    else if (method === 'GET') {
                        res.json().then(function (json) {
                            if (cache) {
                                cached[hashed] = {
                                    body: Object.assign({}, json),
                                    headers: Object.assign({}, toObject(res.headers)),
                                    status: res.status,
                                    ok: res.ok
                                };
                            }
                            resolve({
                                body: json,
                                headers: toObject(res.headers),
                                status: res.status,
                                ok: res.ok
                            });
                        }, function () {
                            reject({
                                body: {},
                                headers: toObject(res.headers),
                                status: res.status,
                                ok: res.ok
                            });
                        });
                    }
                    else {
                        res.json().then(function (json) {
                            resolve({
                                body: json,
                                headers: toObject(res.headers),
                                status: res.status,
                                ok: res.ok
                            });
                        }, function () {
                            resolve({
                                body: {},
                                headers: toObject(res.headers),
                                status: res.status,
                                ok: res.ok
                            });
                        });
                    }
                }, function (err) {
                    reject({
                        body: {},
                        headers: {},
                        status: 0,
                        ok: false,
                        err: err
                    });
                });
            });
        };
    }
    return ApiClient;
}());

var exec = function (_a) {
    var dispatch = _a.dispatch, client = _a.client, url = _a.url, method = _a.method, values = _a.values, start = _a.start, success = _a.success, fail = _a.fail, mode = _a.mode, credentials = _a.credentials, headers = _a.headers, cache = _a.cache, useQuery = _a.useQuery, _b = _a.logger, logger = _b === void 0 ? function () { } : _b, before = _a.before;
    return __awaiter(void 0, void 0, void 0, function () {
        var res, res_1;
        return __generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    before({ values: values, headers: headers });
                    dispatch(start());
                    _c.label = 1;
                case 1:
                    _c.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, client.fetch({
                            url: url,
                            method: method,
                            values: values,
                            mode: mode,
                            credentials: credentials,
                            headers: headers,
                            cache: cache,
                            useQuery: useQuery
                        })];
                case 2:
                    res = _c.sent();
                    dispatch(success(Object.assign({}, { values: values }, res)));
                    return [2 /*return*/, Promise.resolve(Object.assign({}, { values: values }, res))];
                case 3:
                    res_1 = _c.sent();
                    logger(res_1);
                    dispatch(fail(Object.assign({}, { values: values }, res_1)));
                    return [2 /*return*/, Promise.reject(Object.assign({}, { values: values }, res_1))];
                case 4: return [2 /*return*/];
            }
        });
    });
};
var getExecOptions = function (_a) {
    var options = _a.options, values = _a.values, urls = _a.urls, dispatch = _a.dispatch, client = _a.client, resource = _a.resource, action = _a.action, logger = _a.logger, before = _a.before;
    return ({
        dispatch: dispatch,
        client: client,
        url: urls[resource][action].url,
        method: urls[resource][action].method,
        cache: urls[resource][action].cache,
        mode: options.mode !== undefined ? options.mode : urls[resource][action].mode,
        logger: logger,
        credentials: options.credentials !== undefined
            ? options.credentials
            : urls[resource][action].credentials,
        headers: options.headers !== undefined
            ? options.headers
            : urls[resource][action].headers || {},
        useQuery: options.useQuery !== undefined
            ? options.useQuery
            : urls[resource][action].useQuery,
        values: values,
        before: before,
        start: function () { return ({
            values: values,
            type: resource + "/" + snakeCase(action).toUpperCase() + "_START"
        }); },
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        success: function (res) {
            return Object.assign({}, { values: values }, res, {
                type: resource + "/" + snakeCase(action).toUpperCase() + "_SUCCESS"
            });
        },
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        fail: function (res) {
            return Object.assign({}, { values: values }, res, {
                type: resource + "/" + snakeCase(action).toUpperCase() + "_FAIL"
            });
        }
    });
};
var Fetcher = /** @class */ (function () {
    function Fetcher(_a) {
        var _this = this;
        var urls = _a.urls, dispatch = _a.dispatch, _b = _a.headers, headers = _b === void 0 ? {} : _b, _c = _a.logger, logger = _c === void 0 ? function (args) {
            console.log(args);
        } : _c, _d = _a.before, before = _d === void 0 ? function (_a) {
            var values = _a.values, headers = _a.headers;
            return ({ values: values, headers: headers });
        } : _d;
        var client = new ApiClient({
            defaultHeaders: headers,
            logger: logger
        });
        Object.keys(urls).map(function (resource) {
            _this[resource] = {};
            Object.keys(urls[resource]).map(function (action) {
                _this[resource][action] = function (_values, options) {
                    if (options === void 0) { options = {}; }
                    var values = Object.assign({}, urls[resource][action].defaults || {}, _values);
                    _this[resource][action].options = options;
                    return exec(getExecOptions({
                        options: options,
                        values: values,
                        urls: urls,
                        dispatch: dispatch,
                        client: client,
                        resource: resource,
                        action: action,
                        logger: logger,
                        before: before
                    }));
                };
            });
        });
    }
    return Fetcher;
}());

module.exports = Fetcher;
